package com.glxp.mipsdl.admin.client.http;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.glxp.mipsdl.admin.client.BaseHttpClient;
import com.glxp.mipsdl.admin.config.UdPlatConfig;
import com.glxp.mipsdl.admin.entity.ygudplat.UdplatGoodsEntity;
import com.glxp.mipsdl.admin.req.UdiwmsOnhandRequest;
import com.glxp.mipsdl.admin.req.UdiwmsOrderRequest;
import com.glxp.mipsdl.admin.req.UdiwmsProductRequest;
import com.glxp.mipsdl.admin.req.UdiwmsUnitRequest;
import com.glxp.mipsdl.admin.res.system.UdiwmsProductInfoResponse;
import com.glxp.mipsdl.admin.util.Md5Utils;
import com.glxp.mipsdl.common.res.BaseResponse;
import com.glxp.mipsdl.common.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

@Slf4j
@Service
public class UdPlatClient implements BaseHttpClient {

    @Autowired
    private UdPlatConfig udPlatConfig;

    @Override
    public BaseResponse getUnit(UdiwmsUnitRequest testUnitRequest) {
        return null;
    }

    @Override
    public BaseResponse getPrdoucts(UdiwmsProductRequest udiwmsProductRequest) {
        StringBuilder builder = new StringBuilder();
        Map<String, String> params = new TreeMap<>();
        if (null != udiwmsProductRequest) {
            Field[] fields = ReflectUtil.getFields(UdiwmsProductRequest.class);
            for (Field field : fields) {
                Object fieldValue = ReflectUtil.getFieldValue(udiwmsProductRequest, field);
                if (null != fieldValue && "" != fieldValue) {
                    if ("limit".equals(field.getName())) {
                        params.put("pageSize", String.valueOf(fieldValue));
                    } else if ("manufactory".equals(field.getName())) {
                        params.put("manufactureId", String.valueOf(fieldValue));
                    } else if ("name".equals(field.getName())) {
                        params.put("brandName", String.valueOf(fieldValue));
                    } else if ("model".equals(field.getName())) {
                        params.put("model", String.valueOf(fieldValue));
                    } else if ("sepc".equals(field.getName())) {
                        params.put("spec", String.valueOf(fieldValue));
                    } else if ("standard".equals(field.getName())) {
                        params.put("spec", String.valueOf(fieldValue));
                    } else {
                        params.put(field.getName(), String.valueOf(fieldValue));
                    }
                }
            }

            if (CollUtil.isNotEmpty(params)) {
                for (String key : params.keySet()) {
                    if (StrUtil.isBlank(builder.toString())) {
                        builder.append("?").append(key).append("=").append(params.get(key));
                    } else {
                        builder.append("&").append(key).append("=").append(params.get(key));
                    }
                }
            }
        }

        String url = udPlatConfig.getHost() + "/hsapi/purchase/hosp/goods/query";
        url += builder;
        HttpResponse response = HttpRequest.get(url).headerMap(getHeader(params), true).execute();
        List<UdiwmsProductInfoResponse> result = new ArrayList<>();
        if (!response.body().contains("校验失败")) {
            JSONObject jsonObject = JSON.parseObject(response.body());
            List<UdplatGoodsEntity> udplatGoodsEntities = JSON.parseArray(jsonObject.getJSONObject("data").getString("data"), UdplatGoodsEntity.class);
            udplatGoodsEntities = getSingleSpecProduct(udplatGoodsEntities);
            result.addAll(convertProduct(udplatGoodsEntities));
        }
        return ResultVOUtils.success(result);
    }

    /**
     * 查询对照层级为产品的所有规格数据
     *
     * @param udplatGoodsEntities
     */
    private List<UdplatGoodsEntity> getSingleSpecProduct(List<UdplatGoodsEntity> udplatGoodsEntities) {
        List<UdplatGoodsEntity> data = new ArrayList<>();
        for (UdplatGoodsEntity udplatGoodsEntity : udplatGoodsEntities) {
            if (udplatGoodsEntity.getMatchLevel().equals(1)) {
                //对照层级为产品，调用院内目录查询此产品关联的所有规格型号
                Map<String, String> params = new TreeMap<>();
                params.put("hospGoodsId", String.valueOf(udplatGoodsEntity.getId()));
                params.put("compId", udplatGoodsEntity.getCompId());

                String paramUrl = "?hospGoodsId=" + udplatGoodsEntity.getId() + "&compId=" + udplatGoodsEntity.getCompId();
                HttpResponse response = HttpRequest.get(udPlatConfig.getHost() + "/hsapi/purchase/hospGoods/spec/query" + paramUrl).headerMap(getHeader(params), true).execute();
                JSONObject resp = JSON.parseObject(response.body());
                if (resp.getBoolean("success").equals(true)) {
                    JSONArray jsonArray = resp.getJSONObject("data").getJSONArray("data");
                    if (!jsonArray.isEmpty()) {
                        for (Object o : jsonArray) {
                            Map map = JSON.parseObject(JSON.toJSONString(o), Map.class);
                            UdplatGoodsEntity entity = new UdplatGoodsEntity();
                            BeanUtil.copyProperties(udplatGoodsEntity, entity);
                            entity.setSpec(String.valueOf(map.get("spec")));
                            entity.setModel(String.valueOf(map.get("model")));
                            data.add(entity);
                        }
                    }
                }
            } else {
                //单一规格
                data.add(udplatGoodsEntity);
            }
        }
        return data;
    }

    @Override
    public BaseResponse getOrders(UdiwmsOrderRequest udiwmsOrderRequest) {
        return null;
    }

    @Override
    public BaseResponse getInvProducts(UdiwmsOnhandRequest udiwmsOnhandRequest) {
        return null;
    }

    /**
     * 转换产品数据格式
     *
     * @param udplatGoodsEntities
     * @return
     */
    private List<UdiwmsProductInfoResponse> convertProduct(List<UdplatGoodsEntity> udplatGoodsEntities) {
        List<UdiwmsProductInfoResponse> list = new ArrayList<>(udplatGoodsEntities.size());
        for (UdplatGoodsEntity udplatGoodsEntity : udplatGoodsEntities) {
            UdiwmsProductInfoResponse productInfoResponse = new UdiwmsProductInfoResponse();
            BeanUtil.copyProperties(udplatGoodsEntity, productInfoResponse);
            productInfoResponse.setRegisterNo(udplatGoodsEntity.getRegNum());
            productInfoResponse.setYlqxzcrbarmc(udplatGoodsEntity.getRegName());
            productInfoResponse.setSupName(udplatGoodsEntity.getDistributorName());
            productInfoResponse.setManufactory(udplatGoodsEntity.getManufactureName());
            productInfoResponse.setYbbm(udplatGoodsEntity.getMedicalCode());
            productInfoResponse.setZczyxqz(DateUtil.format(udplatGoodsEntity.getRegValidTo(), "YYYY-MM-dd HH:mm:ss"));
            productInfoResponse.setStandard(udplatGoodsEntity.getSpec());
            productInfoResponse.setSpec(udplatGoodsEntity.getSpec() + udplatGoodsEntity.getModel());
            productInfoResponse.setName(udplatGoodsEntity.getProductName());
            list.add(productInfoResponse);
        }
        return list;
    }

    private Map<String, String> getHeader(Map<String, String> mapParam) {
        Map<String, String> map = new TreeMap<>();
        String time = String.valueOf(System.currentTimeMillis());
        mapParam.put("time", time);
        map.put("appId", udPlatConfig.getAppId());
        map.put("Content-Type", "application/json");
        map.put("userName", udPlatConfig.getUserName());
        map.put("time", time);
        map.put("secretKey", udPlatConfig.getSecretKey());
        map.put("sign", getSign(mapParam));
        log.info(map.toString());
        return map;
    }

    /**
     * 获取签名
     *
     * @param mapParam
     * @return
     */
    private String getSign(Map<String, String> mapParam) {
        TreeMap<String, String> map = new TreeMap<>();
        map.put("appId", udPlatConfig.getAppId());
        map.put("userName", udPlatConfig.getUserName());
        map.put("secretKey", udPlatConfig.getUdPlatConfig().getSecretKey());
        map.put("time", mapParam.get("time"));
        map.putAll(mapParam);
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String, String> entry : map.entrySet()) {
            sb.append(entry.getKey()).append("=").append(entry.getValue());
            sb.append("&");
        }
        String str = sb.toString();
        String str0 = str.substring(0, str.length() - 1);
        return Md5Utils.stringToMD5(str0);
    }

}
