package com.glxp.mipsdl.admin.controller;

import com.glxp.mipsdl.admin.client.http.UdPlatClient;
import com.glxp.mipsdl.admin.client.http.YyUCloud31Client;
import com.glxp.mipsdl.admin.http.request.USignRequest;
import com.glxp.mipsdl.admin.req.*;
import com.glxp.mipsdl.common.enums.ResultEnum;
import com.glxp.mipsdl.common.res.BaseResponse;
import com.glxp.mipsdl.common.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@RestController
public class UdiWmsController {

    @Value("${ONLINE_VESRION_FLAG}")
    private String curFlag;
    @Resource
    YyUCloud31Client pzhzdClient;

    @Resource
    private UdPlatClient udPlatClient;

    //获取erp往来单位信息接口
    @PostMapping("/udiwms/erp/getUnits")
    public BaseResponse getUnits(@RequestBody UdiwmsUnitRequest testUnitRequest,
                                 BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (curFlag.equals("YONGYOU_U8_V3.1")) {
            return pzhzdClient.getUnit(testUnitRequest);
        } else
            return ResultVOUtils.error(500, "未设置系统版本！");
    }

    //获取ERP产品信息接口
    @PostMapping("/udiwms/erp/getProducts")
    public BaseResponse getProducts(@RequestBody UdiwmsProductRequest udiwmsProductRequest) {
        if (curFlag.equals("YONGYOU_U8_V3.1")) {
            return pzhzdClient.getPrdoucts(udiwmsProductRequest);
        } else if (curFlag.equals("YG_UDPLAT_V1.0")){
            return udPlatClient.getPrdoucts(udiwmsProductRequest);
        }
            return ResultVOUtils.error(500, "未设置系统版本！");

    }

    //获取ERP订单接口
    @PostMapping("/udiwms/erp/getOrders")
    public BaseResponse getOrders(@RequestBody UdiwmsOrderRequest udiwmsOrderRequest) {
        if (curFlag.equals("YONGYOU_U8_V3.1")) {
            return pzhzdClient.getOrders(udiwmsOrderRequest);
        } else
            return ResultVOUtils.error(500, "未设置系统版本！");

    }

    //获取库存产品基本信息（批次号，生产日期，失效日期，仓位号）
    @PostMapping("/udiwms/erp/inv/getEnvProduct")
    public BaseResponse getEnvProduct(@RequestBody UdiwmsOnhandRequest udiwmsOnhandRequest) {
        if (curFlag.equals("YONGYOU_U8_V3.1")) {
            return pzhzdClient.getInvProducts(udiwmsOnhandRequest);
        } else
            return ResultVOUtils.error(500, "未设置系统版本！");

    }


    //修改ERP订单接口
    @PostMapping("/udiwms/erp/modifyBill")
    public BaseResponse modifyBill(@RequestBody UdiwmsPostBillRequest udiwmsPostBillRequest) {
        if (curFlag.equals("YONGYOU_U8_V3.1")) {
            return pzhzdClient.modifyBill(udiwmsPostBillRequest);
        } else
            return ResultVOUtils.error(500, "未设置系统版本！");

    }

    //签字ERP订单接口
    @PostMapping("/udiwms/erp/signBill")
    public BaseResponse signBill(@RequestBody USignRequest signRequest) {
        if (curFlag.equals("YONGYOU_U8_V3.1")) {
            return pzhzdClient.signBill(signRequest);
        } else
            return ResultVOUtils.error(500, "未设置系统版本！");

    }

    //获取现存量接口
    @GetMapping("/udiwms/erp/getOnhand")
    public BaseResponse getOnHand(@RequestBody UdiwmsOnhandRequest udiwmsOnhandRequest) {
        if (curFlag.equals("YONGYOU_U8_V3.1")) {
            return pzhzdClient.getOnHand(udiwmsOnhandRequest);
        } else
            return ResultVOUtils.error(500, "未设置系统版本！");

    }

}
